/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "id.h"
#include "MyArchive.h"
#include "GeoBase.h"
#include "GeoComment.h"
#include "euclideView.h"
#include "euclideDocument.h"
#include "diaBase.h"

GeoBase::GeoBase( Plan* pln, int idsn ) {

	bPaintOver = FALSE;
	bVisible = TRUE;
	Letter[0] = Letter[1] = Letter[2] = 0;
	Name.loadString( IDSN_BASE );
	Pln = pln;
	Precedent = Suivant = NULL;
	Status = FALSE;
	bExists = FALSE;
	bDefined = FALSE;
	bDestroying = FALSE;
	bMark = FALSE;
	bSelect = FALSE;
	bNoStore = FALSE;
	Width = 1;
	Type = 0;
	bDestroyable = TRUE;
	Name.loadString( idsn );
	Col = idsn - IDSN_OFFSET;
}

GeoBase::~GeoBase() {

	if ( Status ) {

		if ( Precedent ) Precedent->Suivant = Suivant;
		else
			Pln->Racine = Suivant;
		if ( Suivant ) Suivant->Precedent = Precedent;
	}
}

void GeoBase::InvisibleSelected() {

	if (( bSelect )&&( bDestroyable )) bVisible = FALSE;
	if ( Suivant ) Suivant->InvisibleSelected();
}

void GeoBase::Select() {

	bSelect = TRUE;
}

void GeoBase::ChgSelect() {

	bSelect = ! bSelect;
}

void GeoBase::SelectFromParents() {

	if ( Suivant ) Suivant->SelectFromParents();
}

void GeoBase::UnSelect( bool bAll ) {

	bSelect = FALSE;
	if (( bAll )&&( Suivant )) Suivant->UnSelect( TRUE );
}

void GeoBase::ExtendSelectToZone() {

	SelectFromParents();
	if ( Suivant ) Suivant->ExtendSelectToZone();
}

void GeoBase::DispatchNewLetter( char letter ) {

  if (( Type & tpPoint )&&( ! Letter[0] )&&( bSelect )) {
    Letter[0] = letter;
    Letter[1] = 0;
  }
  if ( Suivant ) Suivant->DispatchNewLetter( letter );
}

GeoBase* GeoBase::GetBaseFromIndex( int index ) {

	if ( index == Index ) return this;
	if ( Suivant ) return Suivant->GetBaseFromIndex( index );
	Pln->bErrorStructure = TRUE;
	return NULL;
}

void GeoBase::DoLinkFromIndex() {}

void GeoBase::LinkFromIndex() {

	DoLinkFromIndex();
	if (( Suivant )&&( !Pln->bErrorStructure )) Suivant->LinkFromIndex();
}

void GeoBase::DoStore( CMyArchive& ar ) {}

void GeoBase::Store( CMyArchive& ar ) {

	DoStore( ar );
	if ( Suivant ) Suivant->Store( ar );
}

void GeoBase::DeleteAll() {

	if ( Suivant ) Suivant->DeleteAll();
	delete this;
}

void GeoBase::SetIndex( int i ) {

	if ( bNoStore ) {
		if ( Suivant ) Suivant->SetIndex( i );
		return;
	}

	Index = i;
	if ( Suivant ) Suivant->SetIndex( i+1 );
}

void GeoBase::BurnTheDeads() {

	if ( Suivant ) Suivant->BurnTheDeads();
	if ( bDestroying ) delete this;
}

void GeoBase::Destroy() {

	if (( !bDestroying )&&( bDestroyable )) {
		bDestroying = TRUE;
		Pln->bDestroying = TRUE;
		Pln->Racine->CheckDep();
	}
}

void GeoBase::UnMark() {

	bMark = FALSE;
	if ( Suivant ) Suivant->UnMark();
}

GeoBase* GeoBase::Event( TEvent event, Vect& scrnpoint ) {

  GeoBase* obj = (Suivant)?(Suivant->Event( event, scrnpoint )):NULL;
  
  if (( obj )&&( obj->Type & tpFixPoint )) return obj;
  if (( obj )&&( obj->Type & tpPoint )&&( ! (Type & tpFixPoint) )) return obj;
  
  if ( bExists )
    switch( event ) {
      
    case evLMouse:
    case evRMouse:
    case evLMouseDblClk:
      if ( Inside( scrnpoint )) {
	if ( ! Pln->Status ) view->setMessageText( Name );
	return this;
      }
      break;
    case evLMouseUp:
    case evRMouseUp:
    case evMouseMove:
      break;
    }
  
  return obj;
}

void GeoBase::InvalidateAllOfThem() {

	if ( Status ) bDefined = FALSE;
	if ( Suivant ) Suivant->InvalidateAllOfThem();
}

void GeoBase::BuildDone() {

  Pln->StoreUndo();
  Status = TRUE; bDefined = FALSE; bSelect = TRUE;
  
  Suivant = Pln->Racine;
  Pln->Racine = this;
  if ( Suivant ) Suivant->Precedent = this;
  
  Pln->InvalidateAllOfThem();
  
  Pln->lpstr.loadString( IDS_IDLEMESSAGE );
  view->setMessageText( Pln->lpstr );
}

void GeoBase::Define() {
  
  if ( !bDefined ) DoDefine();
  bDefined = TRUE;
}

void GeoBase::DefineAll() {
  
  Define();
  if ( Suivant ) Suivant->DefineAll();
}

void GeoBase::CheckDep() {

	if ( Suivant ) Suivant->CheckDep();
	DoCheckDep();
}

void GeoBase::Paint() {

	if (( !bPaintOver )&&( bExists )&&( bVisible )) DoPaint();
	if ( Suivant ) Suivant->Paint();
	if (( bPaintOver )&&( bExists )&&( bVisible )) DoPaint();
}

void GeoBase::Properties() {

  diaObject dia( Name, Letter );
  dia.run();
}

void GeoBase::SelectFromZone( Vect p1, Vect p2 ) {

	if ( Suivant ) Suivant->SelectFromZone( p1, p2 );
	DoSelectFromZone( p1, p2 );
}

void GeoBase::DoSelectFromZone( Vect p1, Vect p2 ) {}

Vect GeoBase::Rectify( Vect& p ) {

	return p;
}

void GeoBase::ctSelected() {

	if ( bSelect ) Pln->nSelected++;
	if ( Suivant ) Suivant->ctSelected();
}

void GeoBase::DeleteSelected() {

  if ( bSelect ) Destroy();
  if ( Suivant ) Suivant->DeleteSelected();
}

void GeoBase::HideSelected() {
  
  if ( bSelect ) bVisible = FALSE;
  if ( Suivant ) Suivant->HideSelected();
}

GeoBase* GeoBase::GetSelected() {

	if ( bSelect ) return this;
	else return ( Suivant )?( Suivant->GetSelected() ):( NULL );
}

void GeoBase::SelectAll() {

	bSelect = TRUE;
	if ( Suivant ) Suivant->SelectAll();
}

bool GeoBase::bNeedPoint() {

	return FALSE;
}

//////////////////////////////////////////////////////////////////
// GeoPoint - 

GeoPoint::GeoPoint( Plan* pln, int idsn ):GeoBase( pln, idsn ) {

	bShift = FALSE;
	Type = tpPoint;
}

bool GeoPoint::Inside( Vect& scrnpoint ) {

	if ( !bVisible ) return FALSE;
	Vect p0 = Pln->GetScreenCoords( p );

	if ( (p0-scrnpoint).norm() <= Pln->MouseMarge ) return TRUE;
	return FALSE;
}

void GeoPoint::DoPaint() {

	Vect p0 = Pln->GetDcCoords( p );

	Pln->getDoc()->paintPoint( (int)p0.X[0], (int)p0.X[1], Col, bSelect, bMark );

	if ( Letter )
	  Pln->getDoc()->paintLetter( (int)p0.X[0], (int)p0.X[1] + 12, Col, Letter );
}

void GeoPoint::DoSelectFromZone( Vect p1, Vect p2 ) {

  Vect p0 = Pln->GetDcCoords( p );
  
  if (( p0.X[0] - p1.X[0] )*( p0.X[0] - p2.X[0] ) < 0 ) 
    if ((  p0.X[1] - p1.X[1] )*( p0.X[1] - p2.X[1] ) < 0 ) 
      bSelect = TRUE;
}

//////////////////////////////////////////////////////////////////////
// GeoLine -----------------------------------------------------------
// Basic object for lines  -------------------------------------------

GeoLine::GeoLine( Plan* pln, int idsn ) : GeoBase( pln, idsn ) {

	Type = tpLine;
}

bool GeoLine::Inside( Vect& scrnpoint ) {

	if ( !bVisible ) return FALSE;

	Vect o0 = Pln->GetScreenCoords( o );
	Vect u0 = Pln->GetScreenCoords( u ) 
		- Pln->GetScreenCoords( Vect(0,0,0 ) );
	!u0;

	Vect p1 = ( (scrnpoint-o0) | u0 ) * u0 + o0;
	if ( ( scrnpoint - p1 ).norm() <= Pln->MouseMarge ) return TRUE;
	return FALSE;
}

Vect GeoLine::Rectify( Vect& p ) {

	return o + ( ( p - o )|u )*u;	// Orthogonal projection on the line
}

void GeoLine::DoPaint() {
  
  Vect o0 = Pln->GetDcCoords( o );
  Vect u0 = Pln->GetDcCoords( u ) - Pln->GetDcCoords( Vect(0,0,0) );
  
  Pln->getDoc()->paintLine( o0, u0, Col, bSelect, bMark );
}

//////////////////////////////////////////////////////////////////////
// GeoCircle ---------------------------------------------------------
// Objet de base pour les cercles ------------------------------------

GeoCircle::GeoCircle( Plan* pln, int idsn ) : GeoBase( pln, idsn ) {

	Type = tpCircle;
}

bool GeoCircle::Inside( Vect& scrnpoint ) {

	if ( !bVisible ) return FALSE;

	Vect o0 = Pln->GetScreenCoords( o );
	double r0 = ( Pln->GetScreenCoords( Vect(r,0,0) )
		- Pln->GetScreenCoords( Vect(0,0,0) ) ).X[0];
	
	if ( fabs( ( scrnpoint - o0 ).norm() - r0 ) <= Pln->MouseMarge ) return TRUE;
	return FALSE;
}

Vect GeoCircle::Rectify( Vect& p ) {

	Vect op = p - o;
	return o + r * (!op);				// Projection centrale sur cercle
}

void GeoCircle::DoPaint() {

  Vect o0 = Pln->GetDcCoords( o );
  double r0 = ( Pln->GetDcCoords( Vect(r,0,0) )
		- Pln->GetDcCoords( Vect(0,0,0) ) ).X[0];
  
  if ( r0 > 100000 ) return;
  if ( o0.norm() > 100000 ) return;
  
  Pln->getDoc()->circle( (int)o0.X[0], (int)o0.X[1], (int)r0, Col, bSelect, bMark );
  
  if ( Letter ) 
    Pln->getDoc()->paintLetter( (int)(o0.X[0]+(1/sqrt(2.0))*r0), (int)(o0.X[1]+(1/sqrt(2.0))*r0), Col, Letter );
}


///////////////////////////////////////////////////////////////////////
// GeoFixPoint --------------------------------------------------------

GeoFixPoint::GeoFixPoint( Plan *pln, bool bLimited ):GeoPoint( pln, IDSN_FIXPOINT ) {

	Geo = NULL;
	bRestrict = bLimited;
	bSliding = FALSE;
	Type |= tpFixPoint;
	Status = FALSE;
}

GeoFixPoint::GeoFixPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ):GeoPoint( pln, IDSN_FIXPOINT ) {

	Geo = NULL;
	bSliding = FALSE;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> bRestrict >> Col >> p.X[0] >> p.X[1];
	if ( bRestrict ) ar >> IGeo;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
	Type |= tpFixPoint;
}

GeoFixPoint::GeoFixPoint( Plan *pln, double x, double y ):GeoPoint( pln, IDSN_FIXPOINT ) {

	bSliding = FALSE;
	GeoPoint::p = Pln->GetMathCoords( Vect( x, y, 0 ) );
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
	Type |= tpFixPoint;
	bRestrict = FALSE;
}

void GeoFixPoint::DoLinkFromIndex() {

	if ( bRestrict ) Geo = Pln->Racine->GetBaseFromIndex( IGeo );
}

GeoBase* GeoFixPoint::Event( TEvent event, Vect& scrnpoint ) {

  switch( event ) {
    
  case evLMouse:
    if ( Inside( scrnpoint ) ) {
      
      Vect p0 = Pln->GetScreenCoords( p );
      xSlide = (int)( scrnpoint.X[0]-p0.X[0] );
      ySlide = (int)( scrnpoint.X[1]-p0.X[1] );
      bSliding = TRUE;
      if ( !Pln->Status ) view->setMessageText( Name );
      return this;
    }
    break;
  case evLMouseUp:
    bSliding = FALSE;
    break;
    
  case evMouseMove:
    if ( bSliding ) {
      
      p = Pln->GetMathCoords( 
			     Vect( scrnpoint.X[0] - xSlide,
				   scrnpoint.X[1] - ySlide,
				   0 ) );
      if ( bRestrict ) p = Geo->Rectify( p );
      Pln->InvalidateAllOfThem();
      return this;
    }
    break;
  case evRMouse:
  case evRMouseUp:
  case evLMouseDblClk:
    break;
  }
  
  return GeoPoint::Event( event, scrnpoint );
}

void GeoFixPoint::DoDefine() { 

	bExists = TRUE; 
	if ( bRestrict ) {
		Geo->Define();
		if ( Geo->bExists ) p = Geo->Rectify( p );
		else bExists = FALSE;
	}
} 

void GeoFixPoint::DoCheckDep() {

	if (( bRestrict )&&( Geo->bDestroying )) bRestrict = FALSE;
}

bool GeoFixPoint::AddObject( GeoBase* obj ) {

	if ( bRestrict ) {
		Geo = obj;
		BuildDone();
		if ( bRestrict ) p = Geo->Rectify( p );
		return TRUE;
	}
	return FALSE;
}

bool GeoFixPoint::AddPoint( Vect& scrnpoint ) {

	p = Pln->GetMathCoords( scrnpoint );

	if ( bRestrict ) return FALSE;

	BuildDone();
	return TRUE;
}

void GeoFixPoint::DoStore( CMyArchive& ar ) {

  if ( !Geo ) bRestrict = FALSE;
  ar.setType( IDM_FIXPOINT );
  ar << bVisible << Letter[0] << Letter[1] << bRestrict << Col << p.X[0] << p.X[1];
  if ( bRestrict ) ar << Geo->Index;
}

void GeoFixPoint::Properties() {

  GeoPoint::Properties();
  bSliding = FALSE;   // Prevent the point to be still catched by the mouse after properties.
}

void GeoFixPoint::isRestricted( GeoBase* geo ) {

	bRestrict = TRUE;
	Geo = geo;
}

void GeoFixPoint::Left() {

	p.X[0] -= 0.02;
}

void GeoFixPoint::Right() {

	p.X[0] += 0.02;
}

void GeoFixPoint::Up() {

	p.X[1] -= 0.02;
}

void GeoFixPoint::Down() {

	p.X[1] += 0.02;
}

///////////////////////////////////////////////////////////////////
// GeoPointIntersect ----------------------------------------------
// Intersection de deux droites -----------------------------------

GeoPointIntersect::GeoPointIntersect( Plan *pln ) : GeoPoint( pln, IDSN_POINTINTERSECT ) {

	nLines = 0;
}

GeoPointIntersect::GeoPointIntersect( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_POINTINTERSECT ) {

	nLines = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoPointIntersect::GeoPointIntersect( Plan *pln, GeoLine *_d1, GeoLine *_d2 ):GeoPoint( pln, IDSN_POINTINTERSECT ) {

	d1 = _d1; d2 = _d2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoPointIntersect::DoDefine() {

	if ( d1 ) d1->Define();
	if ( d2 ) d2->Define();
	if (( d1->bExists )&&( d2->bExists )) {

		Vect o1 = d1->o; Vect u1 = d1->u;
		Vect o2 = d2->o - o1; Vect u2 = d2->u;

		double t = (( u2.X[0] * o2.X[1] ) - ( u2.X[1] * o2.X[0] ));
		double den = (( u1.X[0] * u2.X[1] ) - ( u1.X[1] * u2.X[0] ));
		if ( !den ) {
			bExists = FALSE;
			return;
		}

		t = t/den;
		p = o1 - t*u1;
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoPointIntersect::DoCheckDep() {

	if (( d1->bDestroying )||( d2->bDestroying )) Destroy();
}

bool GeoPointIntersect::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpLine ) {

		if ( nLines == 0 ) {
			d1 = (GeoLine*)obj;
			nLines++;
			obj->bMark = TRUE;
		}
		if (( nLines == 1 )&&( d1 != obj )) {
			d2 = (GeoLine*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoPointIntersect::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoPointIntersect::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_POINTINTERSECT );
  ar << bVisible << Letter[0] << Letter[1] << Col << d1->Index << d2->Index;
}

void GeoPointIntersect::DoLinkFromIndex() {

	d1 = (GeoLine*)Pln->Racine->GetBaseFromIndex( I1 );
	d2 = (GeoLine*)Pln->Racine->GetBaseFromIndex( I2 );
}

///////////////////////////////////////////////////////////////////
// GeoMidPoint ----------------------------------------------------
// Milieu de deux points ------------------------------------------

GeoMidPoint::GeoMidPoint( Plan *pln ) : GeoPoint( pln, IDSN_MIDPOINT ) {

	nPoints = 0;
	Suivant = pln->Racine;
}

GeoMidPoint::GeoMidPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_MIDPOINT ) {

	nPoints = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent  = this;
	Status = TRUE;
}

GeoMidPoint::GeoMidPoint( Plan *pln, GeoPoint *_p1, GeoPoint *_p2 ):GeoPoint( pln, IDSN_MIDPOINT ) {

	p1 = _p1; p2 = _p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoMidPoint::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if (( p1->bExists )&&( p2->bExists )) {

		Vect _p1 = p1->p;
		Vect _p2 = p2->p;

		p = (_p1+_p2)*(0.5);
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoMidPoint::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoMidPoint::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoMidPoint::bNeedPoint() {

	return TRUE;
}

bool GeoMidPoint::AddPoint( Vect& scrnpoint ) { return FALSE; }


void GeoMidPoint::DoStore( CMyArchive& ar ) {

    ar.setType( IDM_MIDPOINT );
    ar << bVisible << Letter[0] << Letter[1] << Col << p1->Index << p2->Index;
}

void GeoMidPoint::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

///////////////////////////////////////////////////////////////////
// GeoCarre -------------------------------------------------------
// Image ; rotation PI/2 ------------------------------------------

GeoCarre::GeoCarre( Plan *pln ) : GeoPoint( pln, IDSN_CARRE ) {

	nPoints = 0;
}

GeoCarre::GeoCarre( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_CARRE ) {

	nPoints = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoCarre::GeoCarre( Plan *pln, GeoPoint *_p1, GeoPoint *_p2 ):GeoPoint( pln, IDSN_CARRE ) {

	p1 = _p1; p2 = _p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoCarre::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if (( p1->bExists )&&( p2->bExists )) {

		Vect o = p1->p;
		Vect u = p2->p - o;

		u = u * Vect( 0,0,1 );
		p = o + u;
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoCarre::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoCarre::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoCarre::bNeedPoint() {

	return TRUE;
}

bool GeoCarre::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoCarre ::DoStore( CMyArchive& ar ) {

    ar.setType( IDM_CARRE );
    ar << bVisible << Letter[0] << Letter[1] << Col << p1->Index << p2->Index;
}

void GeoCarre::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoCarre::SelectFromParents() {

	if (( p1->bSelect )||( p2->bSelect )) bSelect = TRUE;
	GeoPoint::SelectFromParents();
}

///////////////////////////////////////////////////////////////////
// GeoVect -------------------------------------------------------
// Image ; translation -------------------------------------------

GeoVect::GeoVect( Plan *pln ) : GeoPoint( pln, IDSN_VECT ) {

	nPoints = 0;
}

GeoVect::GeoVect( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_VECT ) {

	nPoints = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2 >> I3;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoVect::GeoVect( Plan *pln, GeoPoint *_p1, GeoPoint *_p2, GeoPoint *_p3 ):GeoPoint( pln, IDSN_VECT ) {

	p1 = _p1; p2 = _p2; p3 = _p3;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoVect::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if ( p3 ) p3->Define();
	if (( p1->bExists )&&( p2->bExists )&&( p3->bExists )) {

		Vect o = p3->p;
		Vect u = p2->p - p1->p;

		p = o + u;
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoVect::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )||( p3->bDestroying ))
		Destroy();
}

bool GeoVect::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 2 )&&( p2 != obj )) {
			p3 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoVect::bNeedPoint() {

	return TRUE;
}

bool GeoVect::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoVect::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_VECT );
  ar << bVisible << Letter[0] << Letter[1] << Col << p1->Index << p2->Index << p3->Index;
}

void GeoVect::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
	p3 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I3 );
}

///////////////////////////////////////////////////////////////////
// GeoLineAB ------------------------------------------------------
// Droite mene par deux points -----------------------------------

GeoLineAB::GeoLineAB( Plan *pln ) : GeoLine( pln, IDSN_LINEAB ) {

	nPoints = 0;
	bSegment = FALSE;
}

GeoLineAB::GeoLineAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoLine( pln, IDSN_LINEAB ) {

	nPoints = 0;
	Index = index;
	ar >> Width >> Style >> bVisible >> Letter[0] >> Letter[1] >> Col >> bSegment >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoLineAB::GeoLineAB( Plan *pln, GeoPoint *_p1, GeoPoint *_p2, bool _bSegment ):GeoLine( pln, IDSN_LINEAB ) {

	bSegment = _bSegment;
	p1 = _p1; p2 = _p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoLineAB::IsSegment() {

	bSegment = TRUE;
}

bool GeoLineAB::Inside( Vect& scrnpoint ) {

	if ( !bSegment ) {
		return GeoLine::Inside( scrnpoint );
	}
	if ( !GeoLine::Inside( scrnpoint ) ) return FALSE;
	
	Vect p10 = Pln->GetScreenCoords( p1->p );
	Vect p20 = Pln->GetScreenCoords( p2->p );
	Vect u = p20 - p10;

	return ( (( scrnpoint-p10 )|u)  > 0 )&&( (( scrnpoint-p20 )|u) < 0 );

}

Vect GeoLineAB::Rectify( Vect& p ) {

	Vect _p = GeoLine::Rectify( p );

	if ( bSegment ) {

		Vect pp1 = _p - p1->p;
		Vect pp2 = _p - p2->p;

		if (( pp1 | pp2 )<0) return _p;
		if ( pp1.norm2() > pp2.norm2() ) return p2->p;
		return p1->p;
	}
	return _p;
}

void GeoLineAB::DoPaint() {

	if ( ! bSegment ) {
		GeoLine::DoPaint();
		return;
	}

	Vect p10 = Pln->GetDcCoords( p1->p );
	Vect p20 = Pln->GetDcCoords( p2->p );

	Pln->getDoc()->paintSegment( (int)p10.X[0], (int)p10.X[1], (int)p20.X[0], (int)p20.X[1], Col, bSelect, bMark );
}

void GeoLineAB::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if (( p1->bExists )&&( p2->bExists )) {

		o = p1->p;
		u = !( p2->p - p1->p );
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoLineAB::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoLineAB::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoLineAB::bNeedPoint() {

	return TRUE;
}

bool GeoLineAB::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoLineAB::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_LINEAB );
  ar << Width << Style << bVisible << Letter[0] << Letter[1] << Col << bSegment << p1->Index << p2->Index;
}

void GeoLineAB::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoLineAB::Properties() {

	Pln->InvalidateAllOfThem();
}

void GeoLineAB::SelectFromParents() {

	if (( p1->bSelect )||( p2->bSelect )) bSelect = TRUE;
	GeoLine::SelectFromParents();
}

//////////////////////////////////////////////////////////////////////
// GeoOrthogonal - Orthogonal line through one point -----------------
// -------------------------------------------------------------------

GeoOrthogonal::GeoOrthogonal( Plan *pln ) : GeoLine( pln, IDSN_ORTHOGONAL ) {

	pDef = FALSE;
	dDef = FALSE;
}

GeoOrthogonal::GeoOrthogonal( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoLine( pln, IDSN_ORTHOGONAL ) {

	pDef = FALSE;
	dDef = FALSE;
	Index = index;
	ar >> Width >> Style >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoOrthogonal::GeoOrthogonal( Plan *pln, GeoPoint *_p, GeoLine *_d ):GeoLine( pln, IDSN_ORTHOGONAL ) {

	p = _p; d = _d;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoOrthogonal::DoDefine() {

	if ( p ) p->Define();
	if ( d ) d->Define();
	if (( p->bExists )&&( d->bExists )) {

		o = p->p;
		u = !( d->u * Vect(0,0,1) );
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoOrthogonal::DoCheckDep() {

	if (( p->bDestroying )||( d->bDestroying )) Destroy();
}

bool GeoOrthogonal::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {
		
		p = (GeoPoint*)obj;
		obj->bMark = TRUE;
		pDef = TRUE;
	}
	if ( obj->Type & tpLine ) {

		d = (GeoLine*)obj;
		obj->bMark = TRUE;
		dDef = TRUE;
	}
	if (( dDef )&&( pDef )) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoOrthogonal::bNeedPoint() {

	return !pDef;
}

bool GeoOrthogonal::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoOrthogonal::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_ORTHOGONAL );
  ar << Width << Style << bVisible << Letter[0] << Letter[1] << Col << d->Index << p->Index;
}

void GeoOrthogonal::DoLinkFromIndex() {

	d = (GeoLine*)Pln->Racine->GetBaseFromIndex( I1 );
	p = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoOrthogonal::SelectFromParents() {

	if ( p->bSelect ) bSelect = TRUE;
	GeoLine::SelectFromParents();
}

//////////////////////////////////////////////////////////////////////
// GeoParallel - Parallel line through one point ---------------------
// -------------------------------------------------------------------

GeoParallel::GeoParallel( Plan *pln ) : GeoLine( pln, IDSN_PARALLEL ) {

	pDef = FALSE;
	dDef = FALSE;
}

GeoParallel::GeoParallel( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoLine( pln, IDSN_PARALLEL ) {

	Index = index;
	ar >> Width >> Style >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoParallel::GeoParallel( Plan *pln, GeoPoint *_p, GeoLine *_d ):GeoLine( pln, IDSN_PARALLEL ) {

	p = _p; d = _d;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoParallel::DoDefine() {

	if ( p ) p->Define();
	if ( d ) d->Define();
	if (( p->bExists )&&( d->bExists )) {

		o = p->p;
		u = d->u;
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoParallel::DoCheckDep() {

	if (( p->bDestroying )||( d->bDestroying )) Destroy();
}

bool GeoParallel::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {
		
		p = (GeoPoint*)obj;
		obj->bMark = TRUE;
		pDef = TRUE;
	}
	if ( obj->Type & tpLine ) {

		d = (GeoLine*)obj;
		obj->bMark = TRUE;
		dDef = TRUE;
	}
	if (( dDef )&&( pDef )) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoParallel::bNeedPoint() {

	return !pDef;
}

bool GeoParallel::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoParallel::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_PARALLEL ); 
  ar << Width << Style << bVisible << Letter[0] << Letter[1] << Col << d->Index << p->Index;
}

void GeoParallel::DoLinkFromIndex() {

	d = (GeoLine*)Pln->Racine->GetBaseFromIndex( I1 );
	p = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoParallel::SelectFromParents() {

	if ( p->bSelect ) bSelect = TRUE;
	GeoLine::SelectFromParents();
}

//////////////////////////////////////////////////////////////////////
// GeoCircleAB -------------------------------------------------------
// Circle : center and radius ----------------------------------------

GeoCircleAB::GeoCircleAB( Plan *pln ) : GeoCircle( pln, IDSN_CIRCLEAB ) {

	nPoints = 0;
}

GeoCircleAB::GeoCircleAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoCircle( pln, IDSN_CIRCLEAB ) {

	nPoints = 0;
	Index = index;
	ar >> Width >> Style >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoCircleAB::GeoCircleAB( Plan *pln, GeoPoint *_p1, GeoPoint *_p2 ):GeoCircle( pln, IDSN_CIRCLEAB ) {

	p1 = _p1; p2 = _p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoCircleAB::DoDefine() {

	p1->Define();
	p2->Define();
	if (( p1->bExists )&&( p2->bExists )) {

		o = p1->p;
		r = ( p2->p - p1->p ).norm();
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoCircleAB::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoCircleAB::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
		}
	return FALSE;
}

bool GeoCircleAB::bNeedPoint() {

	return TRUE;
}

bool GeoCircleAB::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoCircleAB::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_CIRCLEAB );
  ar << Width << Style << bVisible << Letter[0] << Letter[1] << Col << p1->Index << p2->Index;
}

void GeoCircleAB::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoCircleAB::SelectFromParents() {

	if (( p1->bSelect )||( p2->bSelect )) bSelect = TRUE;
	GeoBase::SelectFromParents();
}

/////////////////////////////////////////////////////////////////////////////////////
// GeoAngle -- 

GeoAngle::GeoAngle( Plan* pln ):GeoBase( pln, IDSN_ANGLE ) {

	Type = tpAngle;
	bOriented = TRUE;
	nPoints = 0;
}

GeoAngle::GeoAngle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ):GeoBase( pln, IDSN_ANGLE ) {

	Type = tpAngle;
	nPoints = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> bOriented >> IO >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoAngle::GeoAngle( Plan *pln, GeoPoint* __o, GeoPoint* __p1, GeoPoint* __p2 ):GeoBase( pln, IDSN_ANGLE ) {

	Type = tpAngle;
	_o = __o; _p1 = __p1; _p2 = __p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
	if ( _o->bShift ) bShift = TRUE;
	else {
		_o->bShift = TRUE;
		bShift = FALSE;
	}
}

bool GeoAngle::Inside( Vect& scrnpoint ) {

	if ( !bVisible ) return FALSE;

	int r = Pln->rAngle;
	Vect o0 = Pln->GetScreenCoords( o );
	if ( bShift ) o0 = o0 + Vect( 1.6*r, r, 0 );
	else o0 = o0 + Vect( -r, r, 0 );
	
	if ( fabs( ( scrnpoint - o0 ).norm() - 0.2*r ) <= Pln->MouseMarge ) return TRUE;
	return FALSE;
}

void GeoAngle::DoPaint() {

  int	r = (int)(Pln->rAngle * (( bShift )?( 0.8 ):( 1 )));
  Vect o0 = Pln->GetDcCoords( o );
  Vect p10 = Pln->GetDcCoords( p1 );
  Vect p20 = Pln->GetDcCoords( p2 );
  
  Pln->getDoc()->paintAngle( o0, p10, p20, bShift, bOriented, bClockwise, alpha, r, Col, bSelect, bMark );
}

void GeoAngle::DoDefine() {

  if ( _o ) _o->Define();
  if ( _p1 ) _p1->Define();
  if ( _p2 ) _p2->Define();
  if (( _o->bExists )&&( _p1->bExists )&&( _p2->bExists )) {
    
    o = _o->p;
    p1 = _p1->p;
    p2 = _p2->p;
    
    // angle computation
    
    double prod = ((p2-o)*(p1-o)).X[2];		// vectorial product
    alpha = acos( (!(p2-o))|(!(p1-o)) );
    if (( bOriented )&&( prod < 0 )) alpha = 2*M_PI - alpha;
    bClockwise = (( ! bOriented )&&( prod < 0 ));

    bExists = TRUE;
  }
  else bExists = FALSE;	
}

void GeoAngle::DoCheckDep() {

	if (( _o->bDestroying )||( _p1->bDestroying )||( _p2->bDestroying )) Destroy();
}

bool GeoAngle::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			_o = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( _o != obj )) {
			_p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 2 )&&( _o != obj )&&( _p1 != obj )) {
			_p2 = (GeoPoint*)obj;
			BuildDone();
			if ( _o->bShift ) bShift = TRUE;
			else {
				_o->bShift = TRUE;
				bShift = FALSE;
			}
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoAngle::bNeedPoint() {

	return TRUE;
}

bool GeoAngle::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoAngle::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_ANGLE );
  ar << bVisible << Letter[0] << Letter[1] << Col << bOriented << _o->Index << _p1->Index << _p2->Index;
}

void GeoAngle::DoLinkFromIndex() {

	_o = (GeoPoint*)Pln->Racine->GetBaseFromIndex( IO );
	_p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	_p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );

	if ( _o->bShift ) bShift = TRUE;
	else {
		_o->bShift = TRUE;
		bShift = FALSE;
	}
}

void GeoAngle::SelectFromParents() {

	if ( _o->bSelect ) bSelect = TRUE; 
	GeoBase::SelectFromParents();
}

///////////////////////////////////////////////////////////////////
// GeoPointRotated ------------------------------------------------
// Transform d'une rotation --------------------------------------

GeoPointRotated::GeoPointRotated( Plan *pln ) : GeoPoint( pln, IDSN_POINT_ROTATED ) {

	Factor = 1.0; Alpha = M_PI/3;
	nPoints = 0;
}

GeoPointRotated::GeoPointRotated( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_POINT_ROTATED ) {

	nPoints = 0;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> Alpha >> Factor >> I1 >> I2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoPointRotated::GeoPointRotated( Plan *pln, GeoPoint *_p1, GeoPoint *_p2, double alpha ):GeoPoint( pln, IDSN_POINT_ROTATED ) {

	Factor = 1.0;
	Alpha = alpha;
	p1 = _p1; p2 = _p2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoPointRotated::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if (( p1->bExists )&&( p2->bExists )) {

		Vect _u = p2->p - p1->p;
		Vect u = _u.X[0]*Vect( cos( Alpha ), -sin( Alpha ), 0 ) +
		  _u.X[1]*Vect( sin(Alpha), cos(Alpha),0 );

		p = p1->p + Factor * u;
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoPointRotated::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoPointRotated::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			BuildDone();
			GetAngle();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoPointRotated::bNeedPoint() {

	return TRUE;
}

bool GeoPointRotated::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoPointRotated::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_POINT_ROTATED );
  ar << bVisible << Letter[0] << Letter[1] << Col << Alpha << Factor << p1->Index << p2->Index;
}

void GeoPointRotated::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoPointRotated::GetAngle() {

  diaRotated dia( Name, Letter, Alpha, Factor );
  dia.run();
  Alpha = dia.getAngle();
  Factor = dia.getFactor();
}

void GeoPointRotated::Properties() {

  Pln->StoreUndo();
  GetAngle();
  Pln->InvalidateAllOfThem();
}

//////////////////////////////////////////////////////////////////////
// GeoProjOrthogonal - Projet orthogonal d'un point sur une droite --
// -------------------------------------------------------------------

GeoProjOrthogonal::GeoProjOrthogonal( Plan *pln, bool _bSymetrique ) : GeoPoint( pln, IDSN_PROJ_ORTHOGONAL ) {

	bSymetrique = _bSymetrique;
	pDef = FALSE;
	dDef = FALSE;
}

GeoProjOrthogonal::GeoProjOrthogonal( Plan *pln, int index, GeoBase* racine, CMyArchive& ar, bool _bSymetrique ) 
  : GeoPoint( pln, IDSN_PROJ_ORTHOGONAL ) {

	bSymetrique = _bSymetrique;
	pDef = FALSE;
	dDef = FALSE;
	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoProjOrthogonal::GeoProjOrthogonal( Plan *pln, GeoPoint *_p, GeoLine *_d ):GeoPoint( pln, IDSN_PROJ_ORTHOGONAL ) {

	bSymetrique = FALSE;
	p = _p; d = _d;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoProjOrthogonal::DoDefine() {

	if ( p ) p->Define();
	if ( d ) d->Define();
	if (( p->bExists )&&( d->bExists )) {

		Vect pp = p->p;
		Vect u = d->u;
		Vect o = d->o;

		GeoPoint::p = o + (( pp-o )|u)*u;

		if ( bSymetrique ) GeoPoint::p += ( GeoPoint::p - pp );

		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoProjOrthogonal::DoCheckDep() {

	if (( p->bDestroying )||( d->bDestroying )) Destroy();
}

bool GeoProjOrthogonal::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {
		
		p = (GeoPoint*)obj;
		obj->bMark = TRUE;
		pDef = TRUE;
	}
	if ( obj->Type & tpLine ) {

		d = (GeoLine*)obj;
		obj->bMark = TRUE;
		dDef = TRUE;
	}
	if (( dDef )&&( pDef )) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoProjOrthogonal::bNeedPoint() {

	return !pDef;
}

bool GeoProjOrthogonal::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoProjOrthogonal::DoStore( CMyArchive& ar ) {

  if ( bSymetrique ) ar.setType( IDM_SYMETRIQUE );
  else	ar.setType( IDM_PROJ_ORTHOGONAL );
  ar << bVisible << Letter[0] << Letter[1] << Col << d->Index << p->Index;
}

void GeoProjOrthogonal::DoLinkFromIndex() {

	d = (GeoLine*)Pln->Racine->GetBaseFromIndex( I1 );
	p = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoProjOrthogonal::SelectFromParents() {

	if ( p->bSelect ) bSelect = TRUE;
	GeoPoint::SelectFromParents();
}
