/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _GEOCOMMENT_H_
#define _GEOCOMMENT_H_

#include "Plane.h"
#include "GeoBase.h"

class GeoComment : public GeoBase  {

public:
	GeoComment( Plan *pln, int idsn );
	GeoComment( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );

	virtual GeoBase*	Event( TEvent event, Vect& scrnpoint );
	virtual bool		Inside( Vect& scrnpoint );
			bool		Inside( int nPt, Vect& scrnpoint );
	virtual void		DoPaint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void DoStore( CMyArchive& ar );
	virtual void DoLinkFromIndex();
	virtual	void Properties();
	virtual void DoSelectFromZone( Vect _p1, Vect _p2 );

protected:
	int	bSliding;
	Vect	p1Sliding;
	Vect	p1;
	int	nPoints;
	string  String;
};

class GeoProjCercle : public GeoPoint {

public:
	GeoProjCercle( Plan *pln );
	GeoProjCercle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoProjCercle( Plan *pln,  GeoPoint *, GeoCircle * );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoPoint *p;
	GeoCircle *c;
	int		Ip, Ic;
	bool	bp, bc;
};

class GeoBissectrice : public GeoLine {

public:
	GeoBissectrice( Plan *pln );
	GeoBissectrice( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoBissectrice( Plan *pln,  GeoPoint*, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p1, *p2, *p3;
	int		I1, I2, I3;
	int		nPoints;
};

class GeoInterCercleDroite : public GeoPoint {

public:
	GeoInterCercleDroite( Plan *pln );
	GeoInterCercleDroite( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoInterCercleDroite( Plan *pln,  GeoLine *, GeoCircle *, bool _bSlave = FALSE );

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoLine *l;
	GeoCircle *c;
	int		Il, Ic;
	bool	bl, bc;
	bool	bSlave;
};

class GeoInterCercleCercle : public GeoPoint {

public:
	GeoInterCercleCercle( Plan *pln );
	GeoInterCercleCercle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoInterCercleCercle( Plan *pln,  GeoCircle *, GeoCircle *, bool _bSlave = FALSE );

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoCircle *c1;
	GeoCircle *c2;
	int		I1, I2;
	int		nCercle;
	bool	bSlave;
};

class GeoBary : public GeoPoint {

public:
	GeoBary( Plan *pln );
	GeoBary( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		Properties();
	virtual void		SelectFromParents();

protected:
	double	  m[6];	
	int		nPoints;	// Number of points
	GeoPoint* p[6];
	int		I[6];

	int		iPoints;	// Number of already defined points
};

class GeoCercleTangent : public GeoCircle {

public:
	GeoCercleTangent( Plan *pln );
	GeoCercleTangent( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoCercleTangent( Plan *pln,  GeoPoint *, GeoLine * );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoLine *l;
	GeoPoint *c;
	int		Il, Ic;
	bool	bl, bc;
};

class GeoProjLine : public GeoPoint {

public:
	GeoProjLine( Plan *pln );
	GeoProjLine( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoProjLine( Plan *pln,  GeoPoint *, GeoLine *, GeoLine * );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoLine *l1;
	GeoLine *l2;
	GeoPoint *c;
	int		Il1, Il2, Ic;
	bool	bl1, bl2, bc;
};

class GeoNorm : public GeoComment {

public:
	GeoNorm( Plan* pln );
	GeoNorm( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoNorm( Plan *pln, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoPaint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		Properties();

	double  Norme;
protected:
	GeoPoint	*_p1, *_p2;
	int			I1, I2;
	int			nPoints;
};

class GeoCercle3 : public GeoCircle {

public:
	GeoCercle3( Plan *pln );
	GeoCercle3( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoCercle3( Plan *pln,  GeoPoint*, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p1, *p2, *p3;
	int		I1, I2, I3;
	int		nPoints;
};


#endif
