#include "id.h"
#include "GeoVector.h"
#include "euclideDocument.h"
#include "euclideView.h"
#include "diaBase.h"

///////////////////////////////////////////////////////////////
// GeoVector -- base class for floating vectors --------------

GeoVector::GeoVector( Plan* pln, int idsn ):GeoBase( pln, idsn ) {

	bSliding = FALSE;
	Type = tpVector;
	p = Vect(0,0,0);
}

bool GeoVector::Inside( Vect& scrnpoint ) {

	bSliding = FALSE;
	if ( !bVisible ) return FALSE;
	Vect p1 = Pln->GetScreenCoords( p );
	Vect v1 = Pln->GetScreenCoords( v ) - Pln->GetScreenCoords( Vect(0,0,0 ) );
	Vect p2 = p1 + v1;
	Vect u = scrnpoint-p1;

	double factor =  u | !v1;
	if (( factor < 0 )||( factor*factor > u.norm2() )) return FALSE;

	Vect proj = factor * v1 + p1;
	if ( (scrnpoint-proj).norm2() <= Pln->MouseMarge ) return TRUE;
	return FALSE;
}

void GeoVector::DoPaint() {
  
  Vect p1 = Pln->GetScreenCoords( p );
  Vect p2 = Pln->GetScreenCoords( p+v );
  Pln->getDoc()->paintSegment( (int)p1.X[0], (int)p1.X[1], (int)p2.X[0], (int)p2.X[1], Col, bSelect, bMark );

  Vect p1p2 = !(p2-p1);
  Vect trans = 8*(p1p2*Vect(0,0,1));
  Vect colin = -15 * p1p2;
  Pln->getDoc()->paintSegment( (int)(p1.X[0]-trans.X[0]), (int)(p1.X[1]-trans.X[1]), 
			       (int)(p1.X[0]+trans.X[0]), (int)(p1.X[1]+trans.X[1]), Col, bSelect, bMark );
  Pln->getDoc()->paintSegment( (int)p2.X[0], (int)p2.X[1], 
			       (int)(p2.X[0]+trans.X[0]+colin.X[0]), 
			       (int)(p2.X[1]+trans.X[1]+colin.X[1]), Col, bSelect, bMark );
  Pln->getDoc()->paintSegment( (int)p2.X[0], (int)p2.X[1], 
			       (int)(p2.X[0]-trans.X[0]+colin.X[0]), 
			       (int)(p2.X[1]-trans.X[1]+colin.X[1]), Col, bSelect, bMark );
  trans = 0.7*trans;
  Pln->getDoc()->paintSegment( (int)(p1.X[0]-trans.X[0]), (int)(p1.X[1]-trans.X[1]), 
			       (int)(p1.X[0]+trans.X[0]), (int)(p1.X[1]+trans.X[1]), Col, bSelect, bMark );

  if ( Letter )
    Pln->getDoc()->paintLetter( (int)p1.X[0], (int)p1.X[1] + 12, Col, Letter );
}

void GeoVector::DoSelectFromZone( Vect p1, Vect p2 ) {
  
  Vect p0 = Pln->GetDcCoords( p );
  
  if (( p0.X[0] - p1.X[0] )*( p0.X[0] - p2.X[0] ) < 0 ) 
    if ((  p0.X[1] - p1.X[1] )*( p0.X[1] - p2.X[1] ) < 0 ) 
      bSelect = TRUE;
}

GeoBase* GeoVector::Event( TEvent event, Vect& scrnpoint ) {

  switch( event ) {
    
  case evLMouse:
    if ( Inside( scrnpoint ) ) {
      
      Vect p0 = Pln->GetScreenCoords( p );
      xSlide = (int)( scrnpoint.X[0]-p0.X[0] );
      ySlide = (int)( scrnpoint.X[1]-p0.X[1] );
      bSliding = TRUE;
      if ( !Pln->Status ) view->setMessageText( Name );
      return this;
    }
    break;
  case evLMouseUp:
    bSliding = FALSE;
    break;
    
  case evMouseMove:
    if ( bSliding ) {
      
      p = Pln->GetMathCoords( 
			     Vect( scrnpoint.X[0] - xSlide,
				   scrnpoint.X[1] - ySlide,
				   0 ) );
      Pln->InvalidateAllOfThem();
      return this;
    }
    break;
  case evRMouse:
  case evRMouseUp:
  case evLMouseDblClk:
    break;
  }
  
  return GeoBase::Event( event, scrnpoint );
}

///////////////////////////////////////////////////////////////////
// GeoVectorAB ----------------------------------------------------
// Floating vector defined by two points  -------------------------

GeoVectorAB::GeoVectorAB( Plan *pln ) : GeoVector( pln, IDSN_VECTORAB ) {

  nPoints = 0;
  Suivant = pln->Racine;
  p = Vect(0,0,0);
}

GeoVectorAB::GeoVectorAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoVector( pln, IDSN_VECTORAB ) {

  nPoints = 0;
  Index = index;
  ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2 >> p.X[0] >> p.X[1];
  Suivant = Pln->Racine;
  if ( Suivant ) Suivant->Precedent  = this;
  Status = TRUE;
}

GeoVectorAB::GeoVectorAB( Plan *pln, GeoPoint *_p1, GeoPoint *_p2 ):GeoVector( pln, IDSN_VECTORAB ) {

  p1 = _p1; p2 = _p2;
  Suivant = pln->Racine;
  if ( Suivant ) Suivant->Precedent = this;
  pln->Racine = this;
  Status = TRUE;
}

void GeoVectorAB::DoDefine() {

  if ( p1 ) p1->Define();
  if ( p2 ) p2->Define();
  if (( p1->bExists )&&( p2->bExists )) {
    
    v = p2->p - p1->p;
    
    bExists = TRUE;
  }
  else bExists = FALSE;
}

void GeoVectorAB::DoCheckDep() {

  if (( p1->bDestroying )||( p2->bDestroying )) Destroy();
}

bool GeoVectorAB::AddObject( GeoBase* obj ) {

  if ( obj->Type & tpPoint ) {
    
    if ( nPoints == 0 ) {
      p1 = (GeoPoint*)obj;
      nPoints++;
      obj->bMark = TRUE;
    }
    if (( nPoints == 1 )&&( p1 != obj )) {
      p2 = (GeoPoint*)obj;
      BuildDone();
      return TRUE;
    }
  }
  return FALSE;
}

bool GeoVectorAB::bNeedPoint() {

	return TRUE;
}

bool GeoVectorAB::AddPoint( Vect& scrnpoint ) { return FALSE; }


void GeoVectorAB::DoStore( CMyArchive& ar ) {

    ar.setType( IDM_VECTORAB );
    ar << bVisible << Letter[0] << Letter[1] << Col << p1->Index << p2->Index << p.X[0] << p.X[1];
}

void GeoVectorAB::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

///////////////////////////////////////////////////////////////////
// GeoVectorSum ---------------------------------------------------
// Floating vector, sum/difference of two other vectors  ----------

GeoVectorSum::GeoVectorSum( Plan *pln, bool _difference ) : GeoVector( pln, IDSN_VECTORSUM ) {

  difference = _difference;
  nPoints = 0;
  Suivant = pln->Racine;
}

GeoVectorSum::GeoVectorSum( Plan *pln, int index, GeoBase* racine, CMyArchive& ar )
 : GeoVector( pln, IDSN_VECTORSUM ) {

  nPoints = 0;
  Index = index;
  ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2 >> difference >> p.X[0] >> p.X[1];
  Suivant = Pln->Racine;
  if ( Suivant ) Suivant->Precedent  = this;
  Status = TRUE;
}

GeoVectorSum::GeoVectorSum( Plan *pln, GeoVector *_v1, GeoVector *_v2, bool _difference ):GeoVector( pln, IDSN_VECTORSUM ) {

  difference = _difference;
  v1 = _v1; v2 = _v2;
  Suivant = pln->Racine;
  if ( Suivant ) Suivant->Precedent = this;
  pln->Racine = this;
  Status = TRUE;
}

void GeoVectorSum::DoDefine() {

  if ( v1 ) v1->Define();
  if ( v2 ) v2->Define();
  if (( v1->bExists )&&( v2->bExists )) {
    
    v = difference?(v2->v - v1->v):(v2->v + v1->v);
    
    bExists = TRUE;
  }
  else bExists = FALSE;
}

void GeoVectorSum::DoCheckDep() {

  if (( v1->bDestroying )||( v2->bDestroying )) Destroy();
}

bool GeoVectorSum::AddObject( GeoBase* obj ) {

  if ( obj->Type & tpVector ) {
    
    if ( nPoints == 0 ) {
      v1 = (GeoVector*)obj;
      nPoints++;
      obj->bMark = TRUE;
    }
    if (( nPoints == 1 )&&( v1 != obj )) {
      v2 = (GeoVector*)obj;
      BuildDone();
      return TRUE;
    }
  }
  return FALSE;
}

bool GeoVectorSum::bNeedPoint() {

	return FALSE;
}

bool GeoVectorSum::AddPoint( Vect& scrnpoint ) { return FALSE; }


void GeoVectorSum::DoStore( CMyArchive& ar ) {

    ar.setType( IDM_VECTORSUM );
    ar << bVisible << Letter[0] << Letter[1] << Col << v1->Index << v2->Index << difference << p.X[0] << p.X[1];
}

void GeoVectorSum::DoLinkFromIndex() {

	v1 = (GeoVector*)Pln->Racine->GetBaseFromIndex( I1 );
	v2 = (GeoVector*)Pln->Racine->GetBaseFromIndex( I2 );
}

//////////////////////////////////////////////////////////////////////
// GeoTranslatedPoint - Point translated by a vector -----------------
// -------------------------------------------------------------------

GeoTranslatedPoint::GeoTranslatedPoint( Plan *pln ) : GeoPoint( pln, IDSN_TRANSLATEDPOINT ) {

	pDef = FALSE;
	vDef = FALSE;
}

GeoTranslatedPoint::GeoTranslatedPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_TRANSLATEDPOINT ) {

	pDef = FALSE;
	vDef = FALSE;
	Index = index;
	ar >> Width >> Style >> bVisible >> Letter[0] >> Letter[1] >> Col >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoTranslatedPoint::GeoTranslatedPoint( Plan *pln, GeoPoint *_p, GeoVector *_v ):GeoPoint( pln, IDSN_TRANSLATEDPOINT ) {

	o = _p; v = _v;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoTranslatedPoint::DoDefine() {

  if ( o ) o->Define();
  if ( v ) v->Define();
  if (( o->bExists )&&( v->bExists )) {
    
    p = o->p + v->v;
    bExists = TRUE;
  }
  else bExists = FALSE;
}

void GeoTranslatedPoint::DoCheckDep() {

  if (( o->bDestroying )||( v->bDestroying )) Destroy();
}

bool GeoTranslatedPoint::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {
		
		o = (GeoPoint*)obj;
		obj->bMark = TRUE;
		pDef = TRUE;
	}
	if ( obj->Type & tpVector ) {

		v = (GeoVector*)obj;
		obj->bMark = TRUE;
		vDef = TRUE;
	}
	if (( vDef )&&( pDef )) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoTranslatedPoint::bNeedPoint() {

  return !pDef;
}

bool GeoTranslatedPoint::AddPoint( Vect& scrnpoint ) {

  return FALSE;
}

void GeoTranslatedPoint::DoStore( CMyArchive& ar ) {
  
  ar.setType( IDM_TRANSLATEDPOINT );
  ar << Width << Style << bVisible << Letter[0] << Letter[1] << Col << v->Index << o->Index;
}

void GeoTranslatedPoint::DoLinkFromIndex() {
  
  v = (GeoVector*)Pln->Racine->GetBaseFromIndex( I1 );
  o = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoTranslatedPoint::SelectFromParents() {

  if ( o->bSelect ) bSelect = TRUE;
  GeoPoint::SelectFromParents();
}

/////////////////////////////////////////////////////////////////////////////////////
// GeoScalaireProd -- 

GeoScalaireProd::GeoScalaireProd( Plan* pln ):GeoComment( pln, IDSN_SCALAIREPROD ) {

	nVectors = 0;
}

GeoScalaireProd::GeoScalaireProd( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ):GeoComment( pln, IDSN_SCALAIREPROD ) {

	nVectors = 0;
	Index = index;
	ar >> bVisible >> Col >> I1 >> I2 >> p1.X[0] >> p1.X[1];
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoScalaireProd::GeoScalaireProd( Plan *pln, GeoVector* vv1, GeoVector* vv2 ):GeoComment( pln, IDSN_SCALAIREPROD ) {

	v1 = vv1; v2 = vv2;
	p1 = Pln->GetFreeRegion();
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoScalaireProd::DoPaint() {

  String = "<";
  char c1[3]; 
  char c2[3];
  c1[0] = v1->Letter[0];  c1[1] = v1->Letter[1]; c1[2] = 0;	
  c2[0] = v2->Letter[0];  c2[1] = v2->Letter[1]; c2[2] = 0;
  if ( c1[0] ) String += c1; else String += '?';
  String += ',';
  if ( c2[0] ) String += c2; else String += '?';
  String += "> = ";
  
  char tmp[64]; 
  sprintf( tmp, "%g", Norm );
  String += tmp;
  
  GeoComment::DoPaint();
}

void GeoScalaireProd::DoDefine() {
  
  if ( v1 ) v1->Define();
  if ( v2 ) v2->Define();
  if (( v1->bExists )&&( v2->bExists )) {
    
    double Unite = ( Pln->Unite )?( Pln->Unite->Norme ):1;
    Norm = (v1->v | v2->v)/(Unite*Unite);
    bExists = TRUE;
  }
  else bExists = FALSE;	
}

void GeoScalaireProd::DoCheckDep() {
  
  if (( v1->bDestroying )||( v2->bDestroying )) Destroy();
}

bool GeoScalaireProd::AddObject( GeoBase* obj ) {
  
  if ( obj->Type & tpVector ) {

    if ( nVectors == 0 ) {
      v1 = (GeoVector*)obj;
      nVectors++;
      obj->bMark = TRUE;
    }
    if (( nVectors == 1 )&&( v1 != obj )) {
      v2 = (GeoVector*)obj;
      obj->bMark = TRUE;
      p1 = Pln->GetFreeRegion();
      BuildDone();
      return TRUE;
    }
  }
  return FALSE;
}

bool GeoScalaireProd::bNeedPoint() {

  return FALSE;
}

bool GeoScalaireProd::AddPoint( Vect& scrnpoint ) {
  
  return FALSE;
}

void GeoScalaireProd::DoStore( CMyArchive& ar ) {
  
  ar.setType( IDM_SCALAIREPROD );
  ar << bVisible << Col << v1->Index << v2->Index << p1.X[0] << p1.X[1];
}

void GeoScalaireProd::DoLinkFromIndex() {
  
  v1 = (GeoVector*)Pln->Racine->GetBaseFromIndex( I1 );
  v2 = (GeoVector*)Pln->Racine->GetBaseFromIndex( I2 );
}
