/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _GEOVECT_H_
#define _GEOVECT_H_

#include "Plane.h"
#include "GeoBase.h"
#include "GeoComment.h"

class GeoVector : public GeoBase  {

public:
	GeoVector( Plan *pln, int idsn );

	virtual GeoBase*	Event( TEvent event, Vect& scrnpoint );
	virtual bool		Inside( Vect& scrnpoint );
	virtual void		DoPaint();
	virtual void		DoSelectFromZone( Vect p1, Vect p2 );

	Vect    v;  // The vector

protected:
	int	bSliding; // Sliding the representation ?
	int     xSlide, ySlide;
	Vect	p; // Origin of the representation

};

class GeoVectorAB : public GeoVector {

public:
	GeoVectorAB( Plan *pln );
	GeoVectorAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoVectorAB( Plan *pln, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction achevée.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoPoint *p1, *p2;
	int	 I1, I2;
	int	 nPoints;
};

class GeoVectorSum : public GeoVector {

public:
	GeoVectorSum( Plan *pln, bool _difference );
	GeoVectorSum( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoVectorSum( Plan *pln, GeoVector*, GeoVector*, bool );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction achevée.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoVector *v1, *v2;
	int	 I1, I2;
	int	 nPoints;
	bool     difference;
};

class GeoTranslatedPoint : public GeoPoint {

public:
	GeoTranslatedPoint( Plan *pln );
	GeoTranslatedPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoTranslatedPoint( Plan *pln, GeoPoint*, GeoVector* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction achevée.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *o; 
	GeoVector  *v;
	int	I1, I2;
	bool	pDef;
	bool	vDef;
};

class GeoScalaireProd : public GeoComment {

public:
	GeoScalaireProd( Plan* pln );
	GeoScalaireProd( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoScalaireProd( Plan *pln, GeoVector*, GeoVector* );
	virtual bool		bNeedPoint();

	virtual void		DoPaint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction is done...
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

	double  Norm;
protected:
	GeoVector	*v1, *v2;
	int		I1, I2;
	int		nVectors;
};

#endif
