/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "MyArchive.h"
#include <iostream>

CMyArchive::CMyArchive() {

  Type = TYPE_MEMORY_EVAL;
  Curs = 0;
  fileVersion = -1;
}

CMyArchive::~CMyArchive() {

  if (( Type == TYPE_MEMORY_STORE )||( Type == TYPE_MEMORY_LOAD )) delete [] Buf;
}

void CMyArchive::fromStream( ifstream& is ) {

  int signature = 0;

  BufLen = 0;
  if ( Type == TYPE_MEMORY_EVAL ) {

    Type = TYPE_MEMORY_LOAD;

    is.read( (char*)(&signature), 4 );
    if ( signature == ECL_FILE_SIGNATURE ) {
    
      is.read( (char*)(&signature), 4 );
      fileVersion = signature;
    
      is.read( (char*)(&BufLen), 4 );
      Buf = new char[BufLen];
      is.read( Buf, BufLen );
      Curs = Buf;
    } else Type = TYPE_CLOSE;
  }
}

void CMyArchive::toStream( ofstream& os ) const {

  int signature = 0;
  if ( Type == TYPE_MEMORY_LOAD ) {
    
    signature = ECL_FILE_SIGNATURE;
    os.write( (char*)(&signature), 4 );
    signature = ECL_FILE_VERSION;
    os.write( (char*)(&signature), 4 );
    os.write( (char*)(&BufLen), 4 );
    os.write( Buf, BufLen );
  }
}

bool CMyArchive::signature( int v ) const {

  return ( (fileVersion != -1)&&( v >= fileVersion ) );
}

bool CMyArchive::IsStoring() const {

  return ( Type == TYPE_MEMORY_STORE );
}

bool CMyArchive::IsLoading() const {

  return ( Type == TYPE_MEMORY_LOAD );
}

bool CMyArchive::IsEvaluating() const {

  return ( Type == TYPE_MEMORY_EVAL );
}

void CMyArchive::CloseEvaluating() {

  if ( Type == TYPE_MEMORY_EVAL ) {

    Type = TYPE_MEMORY_STORE;
    BufLen = (long)Curs;
    Buf = new char[BufLen];
    Curs = Buf;
  }
}

void CMyArchive::CloseStoring() {

  if ( Type == TYPE_MEMORY_STORE ) {
    Type = TYPE_MEMORY_LOAD;
    Curs = Buf;
  }
}

void CMyArchive::RestoreLoading() {

  if (( Type == TYPE_MEMORY_LOAD )||( Type == TYPE_CLOSE )) {
    Type = TYPE_MEMORY_LOAD;
    Curs = Buf;
  }
}

#define ARCHIVE_STORE( type, v ) if ( Type == TYPE_MEMORY_EVAL ) Curs+= sizeof( type ); \
  if ( Type == TYPE_MEMORY_STORE ) { \
    if ( (Curs-Buf) >= BufLen ) { \
      Type = TYPE_MEMORY_LOAD; \
      Curs = Buf; \
      return *this; \
    } \
    *((type*)Curs) = v;\
    Curs+=sizeof( type );}

CMyArchive& CMyArchive::operator << ( char v ) {

  ARCHIVE_STORE( char, v );  
  return *this;
}

CMyArchive& CMyArchive::operator << ( int v ) {

  ARCHIVE_STORE( int, v );
  return *this;
}

CMyArchive& CMyArchive::operator << ( bool v ) {

  ARCHIVE_STORE( char, (char)v );
  return *this;
}

CMyArchive& CMyArchive::operator << ( double v ) {

  ARCHIVE_STORE( double, v );
  return *this;
}

CMyArchive& CMyArchive::operator << ( Vect v ) {

  ARCHIVE_STORE( double, v.X[0] );
  ARCHIVE_STORE( double, v.X[1] );
  ARCHIVE_STORE( double, v.X[2] );
  return *this;
}

CMyArchive& CMyArchive::operator << ( const string& v ) {

  ARCHIVE_STORE( int, v.length()+1 );
  if ( Type == TYPE_MEMORY_EVAL ) Curs += v.length()+1;
  if ( Type == TYPE_MEMORY_STORE ) {
  
    memcpy( Curs, v.c_str(), v.length()+1 );
    Curs += v.length()+1;
  }
  return *this;
}

#define ARCHIVE_LOAD( type, v ) if ( Type == TYPE_MEMORY_LOAD ) { \
  if ( Curs-Buf >= BufLen ) { \
    Type = TYPE_CLOSE; \
    return *this; \
  } \
  v = *((type*)(Curs)); \
  Curs += sizeof(type); }

CMyArchive& CMyArchive::operator >> ( char& v ) {

  ARCHIVE_LOAD( char, v );
  return *this;
}

CMyArchive& CMyArchive::operator >> ( int& v ) {

  ARCHIVE_LOAD( int, v );
  return *this;
}

CMyArchive& CMyArchive::operator >> ( bool& v ) {

  v = 0;
  ARCHIVE_LOAD( char, v );
  return *this;
}

CMyArchive& CMyArchive::operator >> ( double& v ) {

  ARCHIVE_LOAD( double, v );
  return *this;
}

CMyArchive& CMyArchive::operator >> ( Vect& v ) {

  ARCHIVE_LOAD( double, v.X[0] );
  ARCHIVE_LOAD( double, v.X[1] );
  ARCHIVE_LOAD( double, v.X[2] );
  return *this;
}

CMyArchive& CMyArchive::operator >> ( string& v ) {

  int len;
  ARCHIVE_LOAD( int, len );
  v = string( Curs, 0, len-1 );
  Curs += len;
  return *this;
}

void  CMyArchive::setType( int id ) {

  *this << id;
}

int   CMyArchive::getType() {

  int id;
  *this >> id;
  return id;
}

