/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _PLANE_H_
#define _PLANE_H_

#include <gtk/gtk.h>
#include "Vect.h"
#include "UndoBuffer.h"
#include "mlstring.h"

typedef enum{ evLMouse, evRMouse, evLMouseUp, evRMouseUp, evMouseMove, evLMouseDblClk }
	TEvent;

#define CURS_NORMAL 0
#define CURS_POINTER 1
#define CURS_SELECT 2
#define CURS_FIXPOINT 3

#define MARQUE_EUCLIDE (int)0x18759A72
#define MAX_COULEUR 30
#define MIN_MOUSEMARGE 3
#define MAX_MOUSEMARGE 12
#define DEFAULT_MOUSEMARGE 10
#define DEFAULT_RANGLE 20

#define DEFAULT_N_UNDO 15

class GeoBase;
class GeoNorm;
class euclideDocument;

class Plan {
	// Plan is the interface between the screen and the mathematical plane
	// Point 0,0 is the center of the screen

public:
	Plan( euclideDocument* _doc, double szx, double szy );
	~Plan();

	void	SetScreenFrame( long strx, long stry );
	void	Zoom( bool in );

	Vect	GetScreenCoords( Vect mathpoint );
	Vect	GetDcCoords( Vect mathpoint );
	Vect	GetMathCoords( Vect scrnpoint );

	void	AVoid( bool bPrint = FALSE );
	void	Paint();
	void	InvalidateAllOfThem();

	int	MouseEvent( TEvent evt, double x, double y );
	bool	Command( int id );
	int	GetCurs() { return nCurs; }

	void	Store( CMyArchive& ar );
	void	Load( CMyArchive& ar );

	void	LoadOptions();
	void	SaveOptions();
	void	NewObject( int );
	void	SelectFromZone( Vect p1, Vect p2 );
	void    selectAll();
	void	UnSelect();
	void	StoreUndo();
	bool	Undo();
	void	Redo();
	bool	GetUndoState();
	bool	GetRedoState();
	void	PreDraw();
	void	ctSelected();		// Count selected objects
	void	SelectToTarget();
	Vect	GetFreeRegion();
	void    DispatchNewLetter( char letter );
	euclideDocument* getDoc() const {return doc;}
	void    setunmodified() { bModified = FALSE; }

	GeoBase* Racine;
	GeoBase* NewObj;
	mlString lpstr;
	GeoNorm* Unite;

	int	MouseMarge;
	int	rAngle;
	bool	bDegrees;
	bool	bErrorStructure;

	int	nSelected;		// How many objects are selected
	bool	Status;			// TRUE = new object in construction
	bool	bDestroying;
	bool    bModified;              // TRUE = something has changed since last save
	bool    ctrlStatus;             // is <ctrl> pressed ?
	double	strX, strY;

protected:
	void	Copy();

	euclideDocument* doc;
	double	yFreeRegion;

	double	xMin, xMax, yMin, yMax;
	double	szX, szY;
	int	nCurs;

	CUndoBuffer* UndoBuffer;
	GeoBase* TargetObject;
};

#endif
