/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "UndoBuffer.h"

#define BUF_LEN 30000

CUndoBuffer::CUndoBuffer( int _n ) {

	if ( _n > MAX_UNDO_BUFFERS ) n = MAX_UNDO_BUFFERS;
	n = _n;

	for ( i=0 ; i <= n ; i++ ) Buffer[i] = NULL;
	i = 0;
	iBottom = 0;
}

CUndoBuffer::~CUndoBuffer() {

}

int CUndoBuffer::iPlus( int i ) {

	return ( i+1 < n ) ? i+1 : 0;
}

int CUndoBuffer::iMoins( int i ) {

	return ( i-1 >= 0 ) ? i-1 : n-1;
}

CMyArchive* CUndoBuffer::GetNextBuffer() {

  i = iPlus(i);
  if ( i == iBottom ) iBottom = iPlus( iBottom );
  
  if ( Buffer[i] ) delete Buffer[i];
  return Buffer[i] = new CMyArchive();
}


CMyArchive* CUndoBuffer::GetUndoBuffer() {

  if ( i == iBottom ) return NULL;
  
  CMyArchive* buf = Buffer[i];
  i = iMoins( i );
  
  if ( buf ) buf->RestoreLoading();
  else i = iPlus(i);
  return buf;
}

CMyArchive* CUndoBuffer::GetRedoBuffer() {

	i = iPlus(i);

	if (( iPlus(i) == iBottom )||( !Buffer[iPlus(i)] )) {
		i = iMoins(i);
		return NULL;
	}

	CMyArchive* buf = Buffer[iPlus(i)];
	buf->RestoreLoading();
	return buf;
}

bool CUndoBuffer::GetUndoState() {

	return ( (!( i == iBottom ))&&( Buffer[i] ));
}

bool CUndoBuffer::GetRedoState() {

	return ( (!( iPlus(i) == iBottom ))&&( Buffer[iPlus(iPlus(i))] ) );
}

bool CUndoBuffer::OnTheTop() {

	return (( iPlus(i) == iBottom )||( !Buffer[iPlus(i)] ));
}
