/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _DIA_BASE_H_
#define _DIA_BASE_H_

#include <gtk/gtk.h>
#include <string>
#include <cmath>
using namespace std;

class errBase {

 public: 
  errBase( int id );
  errBase( const string& message );
  bool run( bool bAsk = FALSE );
  ~errBase();
  void rt_static( gint arg1 ); 

 protected:
  GtkWidget* dialog;
  string str;
  bool  res;
};

class diaBase {

 public: 
  diaBase( const string& _name );
  virtual ~diaBase();
  void doRun();
  virtual void  rt(); // Called when clicking on OK
  void rt_static( gint arg1 ); 

 protected:

  GtkWidget *dialog;
};

class diaObject : public diaBase {

 public:
  diaObject( const string& _name, char* _letter );
  void run();
  virtual void  rt(); // Called when clicking on OK

 protected:

  char* letter;
  GtkWidget *entry, *entryFrame;
};

class diaComment : public diaBase {

 public:
  diaComment( const string& _name, const string& _comment );
  string run();
  virtual void  rt(); // Called when clicking on OK

 protected:

  string comment;
  GtkWidget *entry, *entryFrame;
};

class diaNorm : public diaObject {

 public:
  diaNorm( const string& _name, char* _letter, bool _bUnite );
  bool    run();
  virtual void  rt(); // Called when clicking on OK

 protected:
  bool bUnite;
  GtkWidget *checkbutton;
};

class diaRotated : public diaObject {

 public:
  diaRotated( const string& _name, char* _letter, double _angle, double _factor );
  void    run();
  virtual void  rt(); // Called when clicking on OK
  double getAngle() const { return angle*M_PI/180; }
  double getFactor() const { return factor; }

 protected:
  float angle, factor;
  GtkWidget *editAngle, *editFactor, *frameAngle, *frameFactor, *hbox;
};

class diaBary : public diaObject {

 public:
  diaBary( const string& _name, char* _letter, bool _bNPoints, int _nPoints, const double* _weight );
  void    run();
  virtual void  rt(); // Called when clicking on OK
  int getNPoints() const { return nPoints; }
  double getWeight( int i ) const { return weight[i]; }

 protected:
  void valueChanged();
  static void callbackValueChanged( GtkSpinButton* spinbutton, gpointer user_data );

  int nPoints;
  bool bNPoints;
  float weight[6];
  GtkWidget *editNPoints, *hbox, *frameNPoints;
  GtkWidget *frameWeight[6], *editWeight[6];
};

class diaColors : public diaBase {

 public:
  diaColors( const string& _name, GdkColor *_colors, int _nColors, int _strOffset );
  virtual ~diaColors();
  void run();
  virtual void  rt(); // Called when clicking on OK

 protected:

  GtkWidget **colorButton;
  int ncolors;
  int strOffset;
  GdkColor *colors;
};

class diaWelcome {

 public: 
  diaWelcome();
  ~diaWelcome();

 private:
  GtkWidget* window;
};

#endif
