/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _EUCLIDE_DOCUMENT_H_
#define _EUCLIDE_DOCUMENT_H_

#include <gtk/gtk.h>
#include "Vect.h"
#include "Plane.h"

#define nColors 32
extern GdkColor colors[nColors]; // defined in euclideMain.cpp
#define black 31
#define white 29
#define lightgrey 30

class euclideDocument {

 public:
  
  euclideDocument() : pln( this, 0, 0 ) {}
  euclideDocument( const string& _name, euclideDocument* sv );
  ~euclideDocument();

  void createObject( guint idObj );
  void selectAll();
  void undo();
  void redo();
  void load( ifstream& is );
  bool save( bool force = FALSE, bool* bError = NULL );
  bool saveAs( const string& _name );
  int  myPageIndex();
  euclideDocument* getFromWidget( GtkWidget* widget );
  bool dispatchLetter( char* letter );

  euclideDocument* getSuivant() const { return suivant; }

  gint configure_event( GtkWidget         *widget,
			GdkEventConfigure *event );
  gint expose_event( GtkWidget      *widget,
		     GdkEventExpose *event );
  gint motion_notify_event( GtkWidget       *widget, 
			    gint            x, 
			    gint            y, 
			    GdkModifierType state );
  gint button_press_event( GtkWidget      *widget,
			   GdkEventButton *event );
  gint button_release_event( GtkWidget      *widget,
			     GdkEventButton *event );
  
  void aVoid();
  void paintPoint( int x, int y, int color, bool bSelected, bool bMarked );
  void paintLine( Vect& o0, Vect& u0, int color, bool bSelected, bool bMark ); 
  void paintSegment( int x1, int y1, int x2, int y2, int color, bool bSelected, bool bMark );  
  void paintAngle( Vect& o, Vect& p1, Vect& p2, bool bShift, bool bOriented, bool bClockwise, double alpha, int r, int color, bool bSelected, bool bMark );  
  void circle( int o0, int o1, int r, int color, bool bSelect, bool bMark );
  void paintLetter( int x, int y, int color, char* letter );
  void paintComment( int x, int y, int color, const string& str, bool bSelect );
  void invalidate();
  void startDragging( double x, double y );
  void moveDragging( double x, double y );
  void endDragging( double x, double y );
  void createCompound( int x1, int y1, int x2, int y2 );
  void exportAsImage( const gchar* filename, gint xRes, bool _black );
  bool preClose();

  euclideDocument *obtSuivant() const { return suivant; }

 protected:

  string getFileTitle( const string& fileName );

  static gint szX;
  static gint szY;

  GtkWidget *widgetName;
  GtkWidget *drawing_area;
  GdkPixmap *pixmap;
  GdkGC     *gc;
  guint     width, height;
  bool      bFileDefined;
  bool      bDragging;
  bool      bAllBlack;
  int       xDrag, yDrag;
  int       x2Drag, y2Drag;
  int       idCompound;

  Plan      pln;

  string    name;
  euclideDocument *suivant, *precedent;
};


#endif
