/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string>
#include <iostream>
#include "diaBase.h"
#include "euclideMain.h"
#include "euclideView.h"
#include "euclideDocument.h"
#include "euclideConf.h"
#include "mlstring.h"
#include "id.h"
#include "binreloc.h"
#include "icon.xpm"

using namespace std;

const char *buttonFiles[] = {"point.png","pointonline.png","","lineintersect.png","linecircleintersect.png","circleintersect.png","",
		       "barycenter.png","center.png","","vector.png","rotatehomothetic.png","projectcircle.png","projectline.png",
		       "projectorthogonal.png","symetric.png","","line.png","segment.png","lineorthogonal.png",
		       "lineparallel.png","bissect.png",
		       "comment.png","","triangle.png","triangleortho.png","triangleisocel.png","triangleequilateral.png",
		       "square.png","rectangle.png","complexcircle.png","parallelogram.png","trapez.png","mark.png","","circle.png",
		       "circletangent.png","","angleoriented.png","angle.png","length.png","","vectorab.png","vectorsum.png",
		       "vectordiff.png","vectortrans.png"};

#define RGB( r,g,b ) { 0, r<<8, g<<8, b<<8 }
GdkColor colors[nColors] = { RGB( 215, 20, 100 ), // 0
			     RGB( 215, 150, 130 ),
			     RGB( 180, 220, 205 ),
			     RGB( 140, 180, 200 ),
			     RGB( 240, 100, 150 ),
			     RGB( 200, 180, 80 ),     // 5
			     RGB( 000, 100, 100 ),
			     RGB( 150, 000, 200 ),
			     RGB( 100, 000, 220 ),
			     RGB( 120, 000, 220 ),
			     RGB( 0,0,0 ),            // 10
			     RGB( 155,0,0 ),
			     RGB( 0,155,0 ),
			     RGB( 0,0,155 ),
			     RGB( 0,155,155 ),
			     RGB( 155,155,0 ),        // 15
			     RGB( 155,0,155 ),
			     RGB( 155,128,0 ),
			     RGB( 0,155,128 ),
			     RGB( 128,0,155 ),
			     RGB( 128,0,0 ),          // 20
			     RGB( 128,128,0 ),
			     RGB( 78,12,30),
			     RGB( 28,129,145),
			     RGB( 155,128,65 ),
			     RGB( 75,245,187 ),       // 25
			     RGB( 200,0,255 ),
			     RGB( 100,200,255 ),
			     RGB( 245,80,200 ),
			     RGB( 255,255,255 ),
			     RGB( 180,180,180 ),      // 30
			     RGB( 0,0,0 ) };         

static int buttonId[46] = {IDM_FIXPOINT, IDM_FIXPOINT_LIMITED, 0, IDM_POINTINTERSECT, IDM_INTER_CERCLE_DROITE,
			   IDM_INTER_CERCLE_CERCLE, 0, IDM_BARY, IDM_MIDPOINT, 0, IDM_VECT, IDM_POINT_ROTATED, IDM_PROJ_CIRCLE,
			   IDM_PROJ_LINE, IDM_PROJ_ORTHOGONAL, IDM_SYMETRIQUE, 0, IDM_LINEAB, IDM_SEGMENT, IDM_ORTHOGONAL, IDM_PARALLEL,
			   IDM_BISSECTRICE, IDM_COMMENT, 0, IDM_TRIANGLE, IDM_TRIANGLE_ORTHOGONAL, IDM_TRIANGLE_ISOCEL,
			   IDM_TRIANGLE_EQUILATERAL, IDM_COMP_CARRE, IDM_RECTANGLE, IDM_COMP_CIRCLE, IDM_PARALLELOGRAM,
			   IDM_TRAPEZE, IDM_MARK, 0, IDM_CIRCLEAB, IDM_CERCLE_TANGENT, 0, IDM_ANGLE, IDM_ANGLE_GEO,
			   IDM_NORM, 0, IDM_VECTORAB, IDM_VECTORSUM, IDM_VECTORDIFF, IDM_TRANSLATEDPOINT };
static int buttonIdsn[46] = {IDSN_FIXPOINT, IDSN_FIXPOINT_LIMITED, 0, IDSN_POINTINTERSECT, IDSN_INTER_CERCLE_DROITE,
			   IDSN_INTER_CERCLE_CERCLE, 0, IDSN_BARY, IDSN_MIDPOINT, 0, IDSN_VECT, IDSN_POINT_ROTATED, IDSN_PROJ_CIRCLE,
			   IDSN_PROJ_LINE, IDSN_PROJ_ORTHOGONAL, IDSN_SYMETRIQUE, 0, 
			   IDSN_LINEAB, IDSN_SEGMENT, IDSN_ORTHOGONAL, IDSN_PARALLEL,
			   IDSN_BISSECTRICE, IDSN_COMMENT, 0, IDSN_TRIANGLE, IDSN_TRIANGLE_ORTHOGONAL, IDSN_TRIANGLE_ISOCEL,
			   IDSN_TRIANGLE_EQUILATERAL, IDSN_COMP_CARRE, IDSN_RECTANGLE, IDSN_COMP_CIRCLE, IDSN_PARALLELOGRAM,
			   IDSN_TRAPEZE, IDSN_MARK, 0, IDSN_CIRCLEAB, IDSN_CERCLE_TANGENT, 0, IDSN_ANGLE, IDSN_ANGLE_GEO,
			   IDSN_NORM, 0, IDSN_VECTORAB, IDSN_VECTORSUM, IDSN_VECTORDIFF, IDSN_TRANSLATEDPOINT };

// Global widgets ------------------

  GtkWidget *window;
  GtkWidget *hpaned;
  GtkWidget *vbox;
  GtkWidget *menu = NULL;
  GtkWidget *hbox;
  GtkWidget *statusbar;
  GtkAccelGroup *accel_group = NULL;
  GtkItemFactory *item_factory;
  GtkWidget *spinX, *checkBlack;

void loadGlobalConfigs() {

  for ( int i = 0 ; i < nColors ; i++ ) {

    char name[128];
    sprintf( name, "%d_red", i );
    colors[i].red = config::getInt( name, colors[i].red  );
    sprintf( name, "%d_green", i );
    colors[i].green = config::getInt( name, colors[i].green  );
    sprintf( name, "%d_blue", i );
    colors[i].blue = config::getInt( name, colors[i].blue  );
  }
}

void saveGlobalConfigs() {

  for ( int i = 0 ; i < nColors ; i++ ) {

    char name[128];
    sprintf( name, "%d_red", i );
    config::setInt( name, colors[i].red  );
    sprintf( name, "%d_green", i );
    config::setInt( name, colors[i].green  );
    sprintf( name, "%d_blue", i );
    config::setInt( name, colors[i].blue  );
  }
}

void rt_open_filename (GtkFileSelection *file_selector, gpointer user_data) {

  string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));
  config::setString( "lastpath", filename.c_str() );
  view->open( gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data)));
}

void rt_save_filename (GtkFileSelection *file_selector, gpointer user_data) {

  string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));
  config::setString( "lastpath", filename.c_str() );
  view->saveAs( gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data)));
}

void rt_export_filename (GtkFileSelection *file_selector, gpointer user_data) {

  string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));
  config::setString( "lastpathexport", filename.c_str() );
  gint xRes = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spinX ) );
  config::setInt( "exportX", xRes );
  view->exportAsImage( gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data)), xRes, 
		       gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( checkBlack ) ));
}

void create_file_selection (const string& title, gpointer rt_func ) {

  GtkWidget *file_selector = gtk_file_selection_new (title.c_str());
  char *lastPath = config::getString( "lastpath" );
  string filename;
  
  if ( lastPath ) filename = lastPath;
  else {
    filename = gbr_find_data_dir("/usr/share");
    filename += "/openeuclide/examples/*.ecl";
  }
  gtk_file_selection_complete ( GTK_FILE_SELECTION (file_selector), filename.c_str() );
  
  g_signal_connect (GTK_OBJECT ((GTK_FILE_SELECTION (file_selector))->ok_button),
		    "clicked",
		    G_CALLBACK (rt_func),
		    (gpointer) file_selector);
  
  /* Ensure that the dialog box is destroyed when the user clicks a button. */
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
			    "clicked",
			    G_CALLBACK (gtk_widget_destroy), 
			    (gpointer) file_selector); 
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
			    "clicked",
			    G_CALLBACK (gtk_widget_destroy),
			    (gpointer) file_selector); 
  
  gtk_widget_show(file_selector);
  gtk_window_set_modal(GTK_WINDOW(file_selector), TRUE);
}

void create_file_export_selection (const string& title, gpointer rt_func ) {

  GtkWidget *file_selector = gtk_file_selection_new (title.c_str());

  GtkWidget *hbox = gtk_hbox_new( FALSE, 4 );
  gtk_box_pack_start( GTK_BOX( GTK_FILE_SELECTION(file_selector)->action_area ), 
  	      hbox, FALSE, FALSE, 4 );
 
  mlString mlXres( IDS_XRESOLUTION );
  gtk_box_pack_start( GTK_BOX( hbox ), gtk_label_new( ((string)mlXres).c_str() ), FALSE, FALSE, 4 );
  spinX = gtk_spin_button_new_with_range( 1, 2048, 1 );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(spinX), config::getInt( "exportX", 1024 ) );
  gtk_box_pack_start( GTK_BOX( hbox ), spinX, FALSE, FALSE, 4 );
  gtk_box_pack_start( GTK_BOX( hbox ), gtk_label_new( "pixels." ), FALSE, FALSE, 4 );
  
  mlString mlBlack( IDS_BLACKEXPORT );
  checkBlack = gtk_check_button_new_with_label(((string)mlBlack).c_str() );
  gtk_box_pack_start( GTK_BOX( GTK_FILE_SELECTION(file_selector)->action_area ), checkBlack, FALSE, FALSE, 4 );

  gtk_widget_show( checkBlack );
  gtk_widget_show_all( hbox );

  char *lastPath = config::getString( "lastpathexport" );
  string filename;

  if ( lastPath ) filename = lastPath;
  else {
    filename = "*.png";
  }
  gtk_file_selection_complete ( GTK_FILE_SELECTION (file_selector), filename.c_str() );

  g_signal_connect (GTK_OBJECT ((GTK_FILE_SELECTION (file_selector))->ok_button),
		    "clicked",
		    G_CALLBACK (rt_func),
		    (gpointer) file_selector);
  
  /* Ensure that the dialog box is destroyed when the user clicks a button. */
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
			    "clicked",
			    G_CALLBACK (gtk_widget_destroy), 
			    (gpointer) file_selector); 
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
			    "clicked",
			    G_CALLBACK (gtk_widget_destroy),
			    (gpointer) file_selector); 
  
  gtk_widget_show(file_selector);
  gtk_window_set_modal(GTK_WINDOW(file_selector), TRUE);
}

void menu_file(gpointer callback_data, guint callback_action, GtkWidget *menu_item) {

  switch (callback_action) {
  case 1: view->create();
    break;
  case 2: {
    mlString titleopen( IDS_TITLEOPEN );
    create_file_selection((string)titleopen, (gpointer)rt_open_filename );
    break; }
  case 3:
    break;
  case 4: view->close();
    break;
  case 5:
    if ( view->save() ) break;
  case 6: {
    mlString titlesave( IDS_TITLESAVEAS );
    create_file_selection((string)titlesave, (gpointer)rt_save_filename );
  }
    break;
  case 7: {
    mlString strColors( IDS_COLORS );
    diaColors( (string)strColors, colors, nColors, IDSN_FIXPOINT ).run();
    gboolean success[nColors];
    gdk_colormap_alloc_colors( gdk_colormap_get_system(), colors, nColors, FALSE, TRUE, success );
    view->aVoid();
    saveGlobalConfigs(); }
    break;
  case 8:
    mlString titleexport( IDS_TITLEEXPORT );
    create_file_export_selection((string)titleexport, (gpointer)rt_export_filename );
    break;
  }
}

void menu_edit(gpointer callback_data, guint callback_action, GtkWidget *menu_item) {

  switch (callback_action) {
  case 1: view->undo();
    break;
  case 2: view->redo();
    break;
  case 3: view->selectAll();
    break;
  }
}

void menu_lang(gpointer callback_data, guint callback_action, GtkWidget *menu_item) {

  errBase dia( IDS_CHANGE_LANG );
  if ( dia.run(TRUE) ) mlString::force( callback_action );
}

void menu_obj(gpointer callback_data, guint callback_action, GtkWidget *menu_item) {

  view->createObject( callback_action );
}

gboolean displayMessage(GtkWidget *widget,
			GdkEventCrossing *event,
			gpointer user_data) {
  
  mlString msg( GPOINTER_TO_INT(user_data) );
  view->pushLocalText( (string)msg );
  return FALSE;
}

gboolean popMessage(GtkWidget *widget,
			GdkEventCrossing *event,
			gpointer user_data) {

  view->popLocalText();
  return FALSE;
}

static GtkWidget* objectButton( gint i ) {

  GtkWidget* button;

  if ( strlen( buttonFiles[i] ) ) {

    string filename = gbr_find_data_dir("/usr/share");
    filename += "/openeuclide/img/"; filename += buttonFiles[i];
    button = gtk_button_new();
    gtk_container_set_border_width ( GTK_CONTAINER(button), 0 ) ;
    GtkWidget* image = gtk_image_new_from_file( filename.c_str() );
    gtk_container_add (GTK_CONTAINER(button), image );
    gtk_widget_show( image );

    g_signal_connect (GTK_OBJECT (button),
		      "enter-notify-event",
		      G_CALLBACK (displayMessage),
		      (gpointer)buttonIdsn[i] );
    g_signal_connect (GTK_OBJECT (button),
		      "leave-notify-event",
		      G_CALLBACK (popMessage),
		      NULL );
   g_signal_connect (GTK_OBJECT (button),
		      "clicked",
		      G_CALLBACK (menu_obj),
		      (gpointer)buttonId[i] );
  } else {

    button = gtk_hseparator_new();  
  }
  gtk_widget_show( button );
  return button;
}

static GtkWidget* buttonColumn1() {

  GtkWidget* vbox = gtk_vbox_new(FALSE, 1);

  for ( int i = 0 ; i < 22 ; i++ )
    gtk_box_pack_start( GTK_BOX(vbox), objectButton(i), FALSE, FALSE, 0 );

  gtk_widget_show( vbox );
  return vbox;
}

static GtkWidget* buttonColumn2() {

  GtkWidget* vbox = gtk_vbox_new(FALSE, 1);

  for ( int i = 22 ; i < 46 ; i++ )
    gtk_box_pack_start( GTK_BOX(vbox), objectButton(i), FALSE, FALSE, 0 );

  gtk_widget_show( vbox );
  return vbox;
}

gint _key_press_event( GtkWidget  *widget,
		       GdkEventKey *event,
		       gpointer obj ) {

  if ( event->keyval == GDK_Delete ) {

    view->createObject( IDM_DELETE );
    return TRUE;
  }
  
  if ( event->state & ( GDK_CONTROL_MASK|GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK|GDK_MOD4_MASK|GDK_MOD5_MASK ) ) return FALSE;
  // we accept only unmodified keys between 'a' and 'z'
  return view->dispatchLetter( event->string );
}

gint _configure_event( GtkWidget         *widget,
		       GdkEventConfigure *event,
		       gpointer obj ) {

  return ((euclideDocument*)obj) -> configure_event( widget, event );
}


gint _expose_event( GtkWidget      *widget,
		    GdkEventExpose *event,
		    gpointer obj ) {

  return ((euclideDocument*)obj) -> expose_event( widget, event );
}


gint _motion_notify_event( GtkWidget *widget,
			   GdkEventMotion *event,
			   gpointer obj ) {

  gint x, y;
  GdkModifierType state;
  
  gdk_window_get_pointer (event->window, &x, &y, &state);

  return ((euclideDocument*)obj) -> motion_notify_event( widget, x, y, state );
}


gint _button_press_event( GtkWidget      *widget,
			  GdkEventButton *event,
			  gpointer obj ) {

  return ((euclideDocument*)obj) -> button_press_event( widget, event );
}

gint _button_release_event( GtkWidget      *widget,
			    GdkEventButton *event,
			    gpointer obj ) {

  return ((euclideDocument*)obj) -> button_release_event( widget, event );
}

gboolean  onClose (GtkWidget *widget,
		   GdkEvent *event,
		   gpointer user_data) {

  return view->onClose();
}

void askForQuit(gpointer callback_data, guint callback_action, GtkWidget *menu_item) {

  if ( !onClose( NULL, NULL, NULL ) )  gtk_widget_destroy( window );
}

void quit () {

  gtk_exit (0);
}

int main( int   argc, 
          char *argv[],
	  char *envp[] ) {

  GError *error;
  gbr_init( &error );
  gtk_init (&argc, &argv);
  //  diaWelcome* welcome = new diaWelcome();

  config::init( envp );
  loadGlobalConfigs();

  mlString::select();

  mlString strWelcome(IDS_WELCOME);
  mlString strTitle(IDS_WIN_TITLE);

  view = new euclideView();

  gboolean success[nColors];
  gdk_colormap_alloc_colors( gdk_colormap_get_system(), colors, nColors, FALSE, TRUE, success );
  
  /* ----------- Main window -------- */

  GdkPixbuf *pixBufIcon = gdk_pixbuf_new_from_xpm_data((const char **)icon_xpm);
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(window), ((string)strTitle).c_str());
  gtk_window_set_icon( GTK_WINDOW(window), pixBufIcon );

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
                      GTK_SIGNAL_FUNC (quit), NULL);

  gtk_widget_set_size_request (window, 500, 400);

  /* -------- The vbox ---------- */

  vbox = gtk_vbox_new(FALSE, 1);
  gtk_container_add (GTK_CONTAINER(window), vbox );

  /* --------- The menu ---------- */

  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW(window), accel_group);
      
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);

  gtk_item_factory_create_items (item_factory, 
      mlString::nbElementsMenu(), mlString::elementsMenu(), NULL);

  gtk_box_pack_start( GTK_BOX(vbox), menu=gtk_item_factory_get_widget (item_factory, "<main>"), FALSE, FALSE, 0 );  

  /* --------- hPanned ----------- */

  hpaned = gtk_hpaned_new ();
  gtk_container_set_border_width (GTK_CONTAINER(hpaned), 5);
  
  gtk_box_pack_start( GTK_BOX(vbox), hpaned, TRUE,TRUE, 0 );

  /* ---------- hbox --------- */

  hbox=gtk_hbox_new(FALSE, 0);
  gtk_paned_add1 (GTK_PANED (hpaned), hbox );

  /* ---------- vboxes --------- */

  gtk_box_pack_start( GTK_BOX(hbox), buttonColumn1(), FALSE, FALSE, 3 );
  gtk_box_pack_start( GTK_BOX(hbox), buttonColumn2(), FALSE, FALSE, 3 );

  /* ------------ Tabbed Notebook ------- */

  gtk_paned_add2 (GTK_PANED (hpaned), view->createNoteBook() );

  /* --------- Status bar ---------- */

  statusbar = gtk_statusbar_new();
  gtk_box_pack_start( GTK_BOX(vbox), statusbar, FALSE, FALSE, 1 );
  gtk_statusbar_push( GTK_STATUSBAR( statusbar ), 0, ((string)strWelcome).c_str() );
  view->attachStatusbar( GTK_STATUSBAR( statusbar ));

  gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
		      (GtkSignalFunc) _key_press_event, NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
		      (GtkSignalFunc) onClose, NULL);
  gtk_widget_set_events (window, GDK_ALL_EVENTS_MASK);
  
  gtk_widget_show_all (window);

  view->create();
  //  delete welcome;
  gtk_main ();

  return 0;
}
