/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include "id.h"
#include "diaBase.h"
#include "euclideView.h"

euclideView *view = NULL;

euclideView::euclideView() {

  blockPainting = FALSE;
  statusbar = NULL;
  documents = NULL;
  notebook = NULL;
  noName = 1;
}

euclideView::~euclideView() {}

euclideDocument* euclideView::getDocument() {

  if (!documents) return NULL;

  return documents->getFromWidget( gtk_notebook_get_nth_page ( 
	   GTK_NOTEBOOK(notebook), gtk_notebook_get_current_page( GTK_NOTEBOOK(notebook) ) ) );
}

void euclideView::detachFirstDocument() {

  if ( documents ) documents = documents->getSuivant();
}

GtkWidget* euclideView::createNoteBook() {

  notebook = gtk_notebook_new();
  gtk_notebook_set_scrollable( GTK_NOTEBOOK(notebook), TRUE );
  gtk_notebook_popup_enable( GTK_NOTEBOOK(notebook) );
  gtk_widget_show( notebook );
  return notebook;
}

void euclideView::create() {

  char tabName[64];

  sprintf( tabName, "NoName%d.ecl", noName++ );

  documents = new euclideDocument( tabName, documents );
}

void euclideView::close() {

  euclideDocument* doc = getDocument();
  if ( doc ) {
    doc->preClose();
    delete doc;
  }
}

void euclideView::undo() {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->undo();
}

void euclideView::redo() {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->redo();
}

void euclideView::aVoid() {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->aVoid();
}

void euclideView::createObject( guint idObj ) {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->createObject( idObj );  
}

void euclideView::attachStatusbar( GtkStatusbar* _statusbar ) {

  bLocalText = FALSE;
  statusbar = _statusbar;
  ctxGeneral = gtk_statusbar_get_context_id( statusbar, "general" );
  ctxLocal = gtk_statusbar_get_context_id( statusbar, "local" );
}

void euclideView::setMessageText( string text ) {

  if ( statusbar ) {

    if ( bLocalText ) {
      gtk_statusbar_pop( statusbar, ctxLocal );
      bLocalText = FALSE;
    }
    gtk_statusbar_pop( statusbar, ctxGeneral );
    gtk_statusbar_push( statusbar, ctxGeneral, text.c_str() );
  }
}

void euclideView::pushLocalText( string text ) {
  
  if ( statusbar ) {
    
    if ( bLocalText ) gtk_statusbar_pop( statusbar, ctxLocal );
    bLocalText = TRUE;
    gtk_statusbar_push( statusbar, ctxLocal, text.c_str() );
  }
}

void euclideView::popLocalText() {

  if (( statusbar )&&( bLocalText )) {
    
    bLocalText = FALSE;
    gtk_statusbar_pop( statusbar, ctxLocal );
  }
}

bool euclideView::open( string fileName ) {
  
  ifstream is; is.open( fileName.c_str(), ios_base::in|ios_base::binary );
  if ( !is.good() ) return FALSE;

  documents = new euclideDocument( fileName, documents );  
  documents->load( is );

  is.close();
  return TRUE;
}

bool euclideView::save() {

  euclideDocument* doc = getDocument();
  if ( doc ) return doc->save();  
  else return TRUE;
}

bool euclideView::saveAs( const string& _name ) {

  euclideDocument* doc = getDocument();
  if ( doc ) return doc->saveAs( _name );
  else return TRUE;
}

bool euclideView::dispatchLetter( char* letter ) {

  euclideDocument* doc = getDocument();
  if ( doc ) return doc->dispatchLetter( letter );
  return FALSE;
}

void euclideView::selectAll() {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->selectAll();
}

gboolean euclideView::onClose() {

  for ( euclideDocument* doc = documents ; doc ; doc = doc->obtSuivant() )
    if ( !doc->preClose() ) return TRUE;
  return FALSE;
}

void euclideView::exportAsImage( const gchar* filename, gint xRes, bool _black ) {

  euclideDocument* doc = getDocument();
  if ( doc ) doc->exportAsImage( filename, xRes, _black );
}
